"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TestScheduler = void 0;
const events_1 = require("events");
const TestSession_1 = require("./TestSession");
class TestScheduler extends events_1.EventEmitter {
    constructor(config, browsers, manager) {
        super();
        this.config = config;
        this.browsers = browsers;
        this.manager = manager;
    }
    async schedule(testRun, sessions) {
        for (const session of sessions) {
            this.manager.updateSession(Object.assign(Object.assign({}, session), { status: TestSession_1.SessionStatuses.SCHEDULED }));
        }
        return this.runScheduled(testRun);
    }
    async runScheduled(testRun) {
        const scheduleTasks = [];
        const it = this.manager.scheduledSessions[Symbol.iterator]();
        while (this.manager.runningSessions.size < this.config.concurrency) {
            const { done, value } = it.next();
            if (done || !value) {
                return;
            }
            const session = this.manager.sessions.get(value);
            if (!session) {
                throw new Error(`Could not find session ${value}`);
            }
            scheduleTasks.push(this.runSession(testRun, session));
        }
        return Promise.all(scheduleTasks);
    }
    runSession(testRun, session) {
        this.manager.updateSession(Object.assign(Object.assign({}, session), { testRun, status: TestSession_1.SessionStatuses.INITIALIZING }));
        setTimeout(() => {
            const updatedSession = this.manager.sessions.get(session.id);
            if (updatedSession.testRun !== testRun) {
                // session reloaded in the meantime
                return;
            }
            if (updatedSession.status === TestSession_1.SessionStatuses.INITIALIZING) {
                this.setSessionTimedout(updatedSession, `Did not receive a start signal from browser after ${this.config.sessionStartTimeout}ms.`);
                return;
            }
            if (updatedSession.status === TestSession_1.SessionStatuses.FINISHED) {
                // The session finished by now
                return;
            }
            setTimeout(() => {
                const updatedSession = this.manager.sessions.get(session.id);
                if (updatedSession.testRun !== testRun) {
                    // session reloaded in the meantime
                    return;
                }
                if (updatedSession.status !== TestSession_1.SessionStatuses.FINISHED) {
                    this.setSessionTimedout(updatedSession, `Browser did not finish within ${this.config.sessionStartTimeout}ms.`);
                }
            }, this.config.sessionFinishTimeout);
        }, this.config.sessionStartTimeout);
        // TODO: Select associated browser instead of iterating all browsers
        for (const browser of this.browsers) {
            return browser.startSession(session);
        }
    }
    setSessionTimedout(session, message) {
        this.emit('session-timed-out', {
            id: session.id,
            result: {
                passed: false,
                logs: [],
                tests: [],
                failedImports: [],
                request404s: new Set(),
                error: { message },
            },
        });
    }
}
exports.TestScheduler = TestScheduler;
//# sourceMappingURL=TestSessionScheduler.js.map