"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Terminal = void 0;
const events_1 = require("events");
const log_update_1 = __importDefault(require("log-update"));
const CLEAR_COMMAND = process.platform === 'win32' ? '\x1B[2J\x1B[0f' : '\x1B[2J\x1B[3J\x1B[H';
const KEYCODES = {
    COMMAND_CONTROL_C: '\u0003',
    COMMAND_CONTROL_D: '\u0004',
};
function buildLogString(entries, serverAddress) {
    let str = '';
    for (const entry of entries) {
        let stringsToAdd;
        let indent;
        if (typeof entry === 'string') {
            stringsToAdd = entry.split('\n');
            indent = 0;
        }
        else {
            stringsToAdd = entry.text.split('\n');
            indent = entry.indent;
        }
        for (const string of stringsToAdd) {
            str += `${' '.repeat(indent)}${string.replace(serverAddress, '')}\n`;
        }
    }
    return str;
}
class Terminal extends events_1.EventEmitter {
    constructor() {
        super(...arguments);
        this.originalFunctions = {};
        this.previousDynamic = [];
        this.started = false;
    }
    start(serverAddress, watchMode) {
        this.serverAddress = new RegExp(serverAddress, 'g');
        // start off with an empty line
        console.log('');
        for (const key of Object.keys(console)) {
            if (typeof console[key] === 'function') {
                this.originalFunctions[key] = console[key];
                console[key] = new Proxy(console[key], {
                    apply: (target, thisArg, argArray) => {
                        // TODO: Remove this when fixed in EDS
                        if (argArray.some((arg) => typeof arg === 'string' && arg.startsWith('[BABEL]'))) {
                            return;
                        }
                        // when a console function is called, clear dynamic logs
                        log_update_1.default.clear();
                        // do regular console log
                        target.apply(thisArg, argArray);
                        // rerender dynamic logs
                        this.relogDynamic();
                    },
                });
            }
        }
        if (watchMode) {
            process.stdin.setRawMode(true);
            process.stdin.resume();
            process.stdin.setEncoding('utf8');
            process.stdin.on('data', (key) => {
                if (key === KEYCODES.COMMAND_CONTROL_C || key === KEYCODES.COMMAND_CONTROL_D) {
                    process.stdin.setRawMode(false);
                    this.emit('kill');
                    return;
                }
                if (key.toUpperCase() === 'D') {
                    this.emit('debug');
                    return;
                }
            });
        }
        this.started = true;
    }
    stop() {
        log_update_1.default.done();
        for (const [key, fn] of Object.entries(this.originalFunctions)) {
            console[key] = fn;
        }
        this.started = false;
    }
    restart() {
        process.stdout.write(CLEAR_COMMAND);
        this.relogDynamic();
    }
    logStatic(entriesOrEntry) {
        const entries = Array.isArray(entriesOrEntry) ? entriesOrEntry : [entriesOrEntry];
        if (entries.length === 0) {
            return;
        }
        console.log(buildLogString(entries, this.serverAddress));
    }
    logDynamic(entriesOrEntry) {
        const entries = Array.isArray(entriesOrEntry) ? entriesOrEntry : [entriesOrEntry];
        if (!this.started) {
            return;
        }
        this.previousDynamic = entries;
        log_update_1.default(buildLogString(entries, this.serverAddress));
    }
    relogDynamic() {
        this.logDynamic(this.previousDynamic);
    }
}
exports.Terminal = Terminal;
//# sourceMappingURL=Terminal.js.map